@@ -37,6 +37,7 @@
 
 #define MAX_CGI_DELAY_TIMER (int)MINUTE / TASK_RUNNER_INTERVAL
 #define DUMMY_BUFFER_SIZE 64 * KILOBYTE
+#define UPLOADED_FILE_BUFFER_SIZE 4096
 
 static int delay_timer = 0;
 
@@ -90,6 +91,25 @@ int fix_crappy_cgi_headers(t_cgi_info *cgi_info) {
 	return 0;
 }
 
+char *find_cgi_header(char *buffer, char *header) {
+	char *pos;
+
+	if ((header == NULL) || (buffer == NULL)) {
+		return NULL;
+	}
+
+	if ((pos = strcasestr(buffer, header)) != NULL) {
+		if (pos == buffer) {
+			return buffer;
+		}
+		if (*(pos - 1) == '\n') {
+			return pos;
+		}
+	}
+
+	return NULL;
+}
+
 /*
  * Load balancer
  * ==============
@@ -288,8 +308,8 @@ t_fcgi_server *find_fcgi_server(t_fcgi_server *fcgi_server, char *id) {
  */
 
 pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
-	int post_pipe[2], html_pipe[2], error_pipe[2], i;
-	char *pos, slash = '/', *run[10], cgi_time[16];
+	int post_pipe[2], html_pipe[2], error_pipe[2], i, handle, bytes_read;
+	char *pos, slash = '/', *run[10], cgi_time[16], buffer[UPLOADED_FILE_BUFFER_SIZE];
 	pid_t cgi_pid;
 
 	do {
@@ -399,15 +419,36 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			cgi_info->from_cgi = html_pipe[0];
 			cgi_info->cgi_error = error_pipe[0];
 
-			/* Send POST data to CGI program.
-			 */
 			if ((session->body != NULL) && (session->content_length > 0)) {
+				/* Send POST data to CGI program.
+				 */
 				if (write_buffer(cgi_info->to_cgi, session->body, session->content_length) == -1) {
 					close(cgi_info->to_cgi);
 					close(cgi_info->from_cgi);
 					close(cgi_info->cgi_error);
 					return -1;
 				}
+			} else if (session->uploaded_file != NULL) {
+				/* Send uploaded file to CGI program.
+				 */
+				if ((handle = open(session->uploaded_file, O_RDONLY)) == -1) {
+					bytes_read = -1;
+				} else {
+					while ((bytes_read = read(handle, buffer, UPLOADED_FILE_BUFFER_SIZE)) > 0) {
+						if (write_buffer(cgi_info->to_cgi, buffer, bytes_read) == -1) {
+							bytes_read = -1;
+							break;
+						}
+					}
+					close(handle);
+				}
+
+				if (bytes_read == -1) {
+					close(cgi_info->to_cgi);
+					close(cgi_info->from_cgi);
+					close(cgi_info->cgi_error);
+					return -1;
+				}
 			}
 	}
 
@@ -576,6 +617,8 @@ int connect_to_fcgi_server(t_connect_to *connect_to) {
 
 int send_fcgi_request(t_session *session, int sock) {
 	t_fcgi_buffer fcgi_buffer;
+	char buffer[UPLOADED_FILE_BUFFER_SIZE];
+	int handle, bytes_read;
 
 	fcgi_buffer.sock = sock;
 	fcgi_buffer.size = 0;
@@ -591,12 +634,30 @@ int send_fcgi_request(t_session *session, int sock) {
 
 	fcgi_buffer.mode = 5;
 	if ((session->body != NULL) && (session->content_length > 0)) {
-		/* Send POST data to CGI program.
+		/* Send POST data to FastCGI program.
 		 */
 		if (send_fcgi_buffer(&fcgi_buffer, session->body, session->content_length) == -1) {
 			return -1;
 		}
+	} else if (session->uploaded_file != NULL) {
+		/* Send uploaded file to FastCGI program
+		 */
+		if ((handle = open(session->uploaded_file, O_RDONLY)) == -1) {
+			return -1;
+		}
+		while ((bytes_read = read(handle, buffer, UPLOADED_FILE_BUFFER_SIZE)) > 0) {
+			if (send_fcgi_buffer(&fcgi_buffer, buffer, bytes_read) == -1) {
+				bytes_read = -1;
+				break;
+			}
+		}
+		close(handle);
+
+		if (bytes_read == -1) {
+			return -1;
+		}
 	}
+
 	if (send_fcgi_buffer(&fcgi_buffer, NULL, 0) == -1) {
 		return -1;
 	}
