@@ -215,44 +215,44 @@ int handle_error(t_session *session, int error_code) {
 	session->vars = error_handler->parameters;
 
 	if ((new_fod = (char*)malloc(session->host->website_root_len + strlen(error_handler->handler) + 4)) == NULL) { /* + 3 for .gz (gzip encoding) */
-		result = 500;
-	} else {
-		if (session->file_on_disk != NULL) {
-			free(session->file_on_disk);
-		}
-		session->file_on_disk = new_fod;
+		return 500;
+	}
 
-		memcpy(session->file_on_disk, session->host->website_root, session->host->website_root_len);
-		strcpy(session->file_on_disk + session->host->website_root_len, error_handler->handler);
+	if (session->file_on_disk != NULL) {
+		free(session->file_on_disk);
+	}
+	session->file_on_disk = new_fod;
 
-		if (get_target_extension(session) == -1) {
-			return 500;
-		}
-		check_target_is_cgi(session);
+	memcpy(session->file_on_disk, session->host->website_root, session->host->website_root_len);
+	strcpy(session->file_on_disk + session->host->website_root_len, error_handler->handler);
 
-		if (session->cgi_type != no_cgi) {
-			result = execute_cgi(session);
+	if (get_target_extension(session) == -1) {
+		return 500;
+	}
+	check_target_is_cgi(session);
+
+	if (session->cgi_type != no_cgi) {
+		result = execute_cgi(session);
 #ifdef ENABLE_XSLT
-		} else if (can_transform_with_xslt(session)) {
-			result = handle_xml_file(session);
+	} else if (can_transform_with_xslt(session)) {
+		result = handle_xml_file(session);
 #endif
-		} else switch (is_directory(session->file_on_disk)) {
-			case error:
-				result = 500;
-				break;
-			case yes:
-				result = 301;
-				break;
-			case no:
-				result = send_file(session);
-				break;
-			case no_access:
-				result = 403;
-				break;
-			case not_found:
-				result = 404;
-				break;
-		}
+	} else switch (is_directory(session->file_on_disk)) {
+		case error:
+			result = 500;
+			break;
+		case yes:
+			result = 301;
+			break;
+		case no:
+			result = send_file(session);
+			break;
+		case no_access:
+			result = 403;
+			break;
+		case not_found:
+			result = 404;
+			break;
 	}
 
 	switch (result) {
@@ -847,7 +847,7 @@ int get_path_info(t_session *session) {
 /* Validate URL
  */
 bool validate_url(t_session *session) {
-	if (valid_uri(session->uri)) {
+	if (valid_uri(session->uri, session->host->allow_dot_files)) {
 		return true;
 	}
 
@@ -875,6 +875,7 @@ int serve_client(t_session *session) {
 	t_host *host_record;
 	t_access access;
 	t_deny_body *deny_body;
+	t_req_method request_method;
 #ifdef ENABLE_TOOLKIT
 	int i;
 	t_toolkit_options options;
@@ -913,21 +914,8 @@ int serve_client(t_session *session) {
 		}
 	}
 
-	if (session->request_method == TRACE) {
-		if (session->binding->enable_trace == false) {
-			return 501;
-		}
-		return handle_trace_request(session);
-	} else if ((session->request_method == PUT) || (session->request_method == DELETE)) {
-		if (session->binding->enable_alter == false) {
-			return 501;
-		}
-	} else if (session->request_method == unsupported) {
-		return 501;
-	} else if (session->request_method == unknown) {
-		return 400;
-	}
-
+	/* Find host record
+	 */
 	if (session->hostname != NULL) {
 		remove_port_from_hostname(session->hostname, session->binding);
 
@@ -940,6 +928,29 @@ int serve_client(t_session *session) {
 	}
 	session->host->access_time = session->time;
 
+	/* Actions based on request method
+	 */
+	switch (session->request_method) {
+		case TRACE:
+			if (session->binding->enable_trace == false) {
+				return 501;
+			}
+			return handle_trace_request(session);
+		case PUT:
+		case DELETE:
+			if ((session->binding->enable_alter == false) && (session->host->webdav_app == false)) {
+				return 501;
+			}
+			break;
+		case unknown:
+			return 400;
+		case unsupported:
+			if (session->host->webdav_app == false) {
+				return 501;
+			}
+			break;
+	}
+
 	/* Validate URL
 	 */
 	if (validate_url(session) == false) {
@@ -1018,10 +1029,10 @@ int serve_client(t_session *session) {
 	 */
 #ifdef ENABLE_SSL
 	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit,
-	                     session->binding->use_ssl, session->headerfields);
+	                     session->binding->use_ssl, session->host->allow_dot_files, session->headerfields);
 #else
 	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit,
-	                     false, session->headerfields);
+	                     false, session->host->allow_dot_files, session->headerfields);
 #endif
 
 	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
@@ -1152,7 +1163,7 @@ int serve_client(t_session *session) {
 			session->uri_is_dir = true;
 			break;
 		case no:
-			if ((session->request_method != PUT) && (session->host->enable_path_info)) {
+			if (((session->request_method != PUT) || session->host->webdav_app) && (session->host->enable_path_info)) {
 				if ((result = get_path_info(session)) != 200) {
 					return result;
 				}
@@ -1184,11 +1195,20 @@ int serve_client(t_session *session) {
 		return 500;
 	}
 
-	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
+	if (((session->request_method != PUT) && (session->request_method != DELETE)) || session->host->webdav_app) {
 		check_target_is_cgi(session);
 	}
 
-	switch (session->request_method) {
+	/* Handle request based on request method
+	 */
+	request_method = session->request_method;
+	if (session->host->webdav_app) {
+		if ((request_method == PUT) || (request_method == DELETE)) {
+			request_method = POST;
+		}
+	}
+
+	switch (request_method) {
 		case GET:
 		case HEAD:
 			if (session->cgi_type != no_cgi) {
@@ -1222,6 +1242,7 @@ int serve_client(t_session *session) {
 			}
 			break;
 		case POST:
+		case unsupported:
 			if (session->cgi_type != no_cgi) {
 				result = execute_cgi(session);
 #ifdef ENABLE_XSLT
@@ -1232,9 +1253,6 @@ int serve_client(t_session *session) {
 				result = 405;
 			}
 			break;
-		case OPTIONS:
-			result = handle_options_request(session);
-			break;
 		case PUT:
 			result = handle_put_request(session);
 			if (((result == 201) || (result == 204)) && (session->host->run_on_alter != NULL)) {
@@ -1306,7 +1324,7 @@ void handle_request_result(t_session *session, int result) {
 			}
 			break;
 		case ec_SOCKET_WRITE_ERROR:
-			log_request(session, session->return_code);
+			log_request(session);
 			break;
 		case ec_FORCE_QUIT:
 			log_system(session, "Client kicked");
@@ -1323,7 +1341,7 @@ void handle_request_result(t_session *session, int result) {
 			}
 			session->return_code = 409;
 			send_code(session);
-			log_request(session, session->return_code);
+			log_request(session);
 			break;
 		case ec_INVALID_URL:
 			if ((session->config->ban_on_invalid_url > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
@@ -1414,7 +1432,7 @@ void handle_request_result(t_session *session, int result) {
 	}
 
 	if ((result > 0) && (result != 400)) {
-		log_request(session, result);
+		log_request(session);
 	} else {
 		session->keep_alive = false;
 	}
