@@ -170,7 +170,7 @@ void log_error(t_session *session, char *mesg) {
 
 /* Log a HTTP request.
  */
-void log_request(t_session *session, int code) {
+void log_request(t_session *session) {
 	char str[BUFFER_SIZE + 1], timestamp[TIMESTAMP_SIZE], ip_address[IP_ADDRESS_SIZE];
 	char *user, *field, *uri, *vars, *path_info;
 	t_headerfield *headerfield;
@@ -215,7 +215,7 @@ void log_request(t_session *session, int code) {
 			user = "";
 		}
 
-		snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", code, (long long)session->bytes_sent, user, secure_string(session->method), uri);
+		snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", session->return_code, (long long)session->bytes_sent, user, secure_string(session->method), uri);
 		offset += strlen(str + offset);
 
 		if ((offset < BUFFER_SIZE) && (path_info != NULL)) {
@@ -269,7 +269,7 @@ void log_request(t_session *session, int code) {
 			offset += strlen(str + offset);
 		}
 		if (offset < BUFFER_SIZE) {
-			snprintf(str + offset, BUFFER_SIZE - offset, " %s\" %d %lld", secure_string(session->http_version), code, (long long)session->bytes_sent);
+			snprintf(str + offset, BUFFER_SIZE - offset, " %s\" %d %lld", secure_string(session->http_version), session->return_code, (long long)session->bytes_sent);
 		}
 
 		if (session->config->log_format == extended) {
