@@ -32,6 +32,7 @@
 #define MAX_LENGTH_CONFIGLINE   512
 #define MAX_CACHE_SIZE           50
 #define MAX_UPLOAD_SIZE         100
+#define MONITOR_HOSTNAME      "monitor"
 
 enum t_section { syntax_error = -1, none, binding, virtual_host, directory, fcgi_server
 #ifdef ENABLE_TOOLKIT
@@ -68,6 +69,7 @@ static t_host *new_host(void) {
 #ifdef ENABLE_XSLT
 	host->show_index         = NULL;
 #endif
+	host->allow_dot_files    = false;
 	host->use_gz_file        = false;
 	host->access_list        = NULL;
 	host->alter_list         = NULL;
@@ -105,6 +107,7 @@ static t_host *new_host(void) {
 	init_charlist(&(host->fast_cgi));
 	host->secure_url         = true;
 	host->deny_body          = NULL;
+	host->webdav_app         = false;
 #ifdef ENABLE_MONITOR
 	host->monitor_stats      = NULL;
 	host->monitor_requests   = false;
@@ -558,7 +561,7 @@ int check_configuration(t_config *config) {
 
 	if ((config->monitor_enabled) && (config->first_host->next != NULL)) {
 		host = config->first_host->next;
-		if (strcmp(host->hostname.item[0], "monitor") == 0) {
+		if (strcmp(host->hostname.item[0], MONITOR_HOSTNAME) == 0) {
 			host->website_root = config->monitor_directory;
 			host->website_root_len = strlen(host->website_root);
 		}
@@ -836,7 +839,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		monitor_host->next = config->first_host->next;
 		config->first_host->next = monitor_host;
 
-		if (parse_charlist("monitor", &(monitor_host->hostname)) == -1) {
+		if (parse_charlist(MONITOR_HOSTNAME, &(monitor_host->hostname)) == -1) {
 			return false;
 		}
 
@@ -1077,7 +1080,7 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 			return true;
 		}
 	} else if (strcmp(key, "setenv") == 0) {
-		if (parse_keyvaluelist(value, &(host->envir_str), "=") != -1) {
+		if (parse_keyvalue(value, &(host->envir_str), "=") != -1) {
 			if (register_tempdata(tempdata, host->envir_str, tc_keyvalue) != -1) {
 				return true;
 			} else {
@@ -1134,11 +1137,15 @@ static bool host_setting(char *key, char *value, t_host *host) {
 			}
 		}
 	} else if (strcmp(key, "alias") == 0) {
-		if (parse_keyvaluelist(value, &(host->alias), ":") != -1) {
+		if (parse_keyvalue(value, &(host->alias), ":") != -1) {
 			if (valid_directory(host->alias->key) && valid_directory(host->alias->value)) {
 				return true;
 			}
 		}
+	} else if (strcmp(key, "allowdotfiles") == 0) {
+		if (parse_yesno(value, &(host->allow_dot_files)) == 0) {
+			return true;
+		}
 	} else if (strcmp(key, "denybody") == 0) {
 		if (host->deny_body == NULL) {
 			host->deny_body = (t_deny_body*)malloc(sizeof(t_deny_body));
@@ -1190,7 +1197,13 @@ static bool host_setting(char *key, char *value, t_host *host) {
 			return true;
 		}
 	} else if (strcmp(key, "hostname") == 0) {
-		if (parse_charlist(strlower(value), &(host->hostname)) == 0) {
+		strlower(value);
+#ifdef ENABLE_MONITOR
+		if (strcmp(value, MONITOR_HOSTNAME) == 0) {
+			return false;
+		}
+#endif
+		if (parse_charlist(value, &(host->hostname)) == 0) {
 			return true;
 		}
 	} else if (strcmp(key, "imagereferer") == 0) {
@@ -1269,6 +1282,14 @@ static bool host_setting(char *key, char *value, t_host *host) {
 				}
 			}
 		}
+	} else if (strcmp(key, "webdavapp") == 0) {
+		if (parse_yesno(value, &(host->webdav_app)) == 0) {
+			if (host->webdav_app) {
+				host->execute_cgi = true;
+				host->enable_path_info = true;
+			}
+			return true;
+		}
 	} else if (strcmp(key, "websiteroot") == 0) {
 		if (valid_directory(value)) {
 			if ((host->website_root = strdup(value)) != NULL) {
@@ -1364,7 +1385,7 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 			return true;
 		}
 	} else if (strcmp(key, "setenv") == 0) {
-		if (parse_keyvaluelist(value, &(directory->envir_str), "=") != -1) {
+		if (parse_keyvalue(value, &(directory->envir_str), "=") != -1) {
 			return true;
 		}
 #ifdef ENABLE_XSLT
@@ -1729,10 +1750,10 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 			} else if (split_configline(key, &key, &value) != -1) {
 				/* Configuration option
 				 */
-				key = strlower(key);
+				strlower(key);
 
 				if (strcmp(key, "set") == 0) {
-					if (parse_keyvaluelist(key + 4, &variables, "=") == -1) {
+					if (parse_keyvalue(key + 4, &variables, "=") == -1) {
 						retval = counter;
 					}
 				} else if (strcmp(key, "include") == 0) {
