@@ -190,6 +190,7 @@ typedef struct type_host {
 #ifdef ENABLE_XSLT
 	char            *show_index;
 #endif
+	bool            allow_dot_files;
 	bool            use_gz_file;
 	char            *login_message;
 	char            *passwordfile;
@@ -228,6 +229,7 @@ typedef struct type_host {
 	bool            secure_url;
 	t_charlist      fast_cgi;
 	t_deny_body     *deny_body;
+	bool            webdav_app;
 
 #ifdef ENABLE_MONITOR
 	t_monitor_host_stats *monitor_stats;
