@@ -63,7 +63,6 @@
 #define NEW_FILE -1
 
 char *hs_chunked = "Transfer-Encoding: chunked\r\n";
-char *hs_allow   = "Allow: GET, HEAD, OPTIONS";
 char *hs_post    = ", POST";
 char *hs_trace   = ", TRACE";
 char *hs_alter   = ", PUT, DELETE";
@@ -790,7 +789,7 @@ int execute_cgi(t_session *session) {
 							}
 
 							if (session->expires > -1) {
-								if (strcasestr(cgi_info.input_buffer, "\nExpires:") != NULL) {
+								if (find_cgi_header(cgi_info.input_buffer, "Expires:") != NULL) {
 									session->expires = -1;
 								}
 							}
@@ -799,7 +798,7 @@ int execute_cgi(t_session *session) {
 							/* Look for store-in-cache CGI header
 							 */
 							if (session->request_method == GET) {
-								if ((str_begin = strcasestr(cgi_info.input_buffer, "\nX-Hiawatha-Cache:")) != NULL) {
+								if ((str_begin = find_cgi_header(cgi_info.input_buffer, "X-Hiawatha-Cache:")) != NULL) {
 									str_begin += 18;
 									while (*str_begin == ' ') {
 										str_begin++;
@@ -829,7 +828,7 @@ int execute_cgi(t_session *session) {
 
 							/* Look for remove-from-cache CGI header
 							 */
-							if ((str_begin = strcasestr(cgi_info.input_buffer, "\nX-Hiawatha-Cache-Remove:")) != NULL) {
+							if ((str_begin = find_cgi_header(cgi_info.input_buffer, "X-Hiawatha-Cache-Remove:")) != NULL) {
 								str_begin += 25;
 								while (*str_begin == ' ') {
 									str_begin++;
@@ -851,15 +850,23 @@ int execute_cgi(t_session *session) {
 							}
 #endif
 
-							if (strcasestr(cgi_info.input_buffer, "\nLocation:") != NULL) {
+							if (find_cgi_header(cgi_info.input_buffer, "Location:") != NULL) {
 								session->return_code = 302;
-								if (send_header(session) == -1) {
-									retval = rs_DISCONNECT;
-									break;
-								}
 							} else if ((code = strcasestr(cgi_info.input_buffer, "Status: ")) != NULL) {
 								line = code += 8;
 
+								/* Work-around for buggy Status headers
+								 */
+								if (strncmp(line, "HTTP/", 5) == 0) {
+									line += 5;
+									while ((*line != '\0') && (*line != ' ')) {
+										line++;
+									}
+									if (*line == ' ') {
+										code = ++line;
+									}
+								}
+
 								result = -1;
 								while (*line != '\0') {
 									if ((*line == '\r') || (*line == ' ')) {
@@ -874,14 +881,13 @@ int execute_cgi(t_session *session) {
 									line++;
 								}
 
-								if (result <= 0) {
+								if ((result <= 0) || (result > 999)) {
 									log_error(session, "invalid status code received from CGI");
 								} else if (result != 200) {
+									session->return_code = result;
 									if (session->host->trigger_on_cgi_status) {
 										retval = result;
 										break;
-									} else {
-										session->return_code = result;
 									}
 								}
 							}
@@ -891,14 +897,6 @@ int execute_cgi(t_session *session) {
 								break;
 							}
 
-							if (session->return_code == 401) {
-								if (session->host->auth_method == basic) {
-									send_basic_auth(session);
-								} else {
-									send_digest_auth(session);
-								}
-							}
-
 							if ((strcasestr(cgi_info.input_buffer, hs_conlen) != NULL) || (session->keep_alive == false)) {
 								send_in_chunks = false;
 							} else if (send_buffer(session, hs_chunked, 28) == -1) {
@@ -1032,74 +1030,8 @@ int execute_cgi(t_session *session) {
 	return retval;
 }
 
-/* Send the result of an OPTIONS request method to the client.
+/* Handle TRACE requests
  */
-int handle_options_request(t_session *session) {
-	int code = 200, handle;
-
-	if ((handle = open(session->file_on_disk, O_RDONLY)) == -1) {
-		if (errno == EACCES) {
-			log_error(session, fb_filesystem);
-			code = 403;
-		} else if (session->uri_is_dir) {
-			code = 501;
-		} else {
-			code = 404;
-		}
-	} else {
-		close(handle);
-	}
-	if (session->host->follow_symlinks == false) {
-		switch (contains_not_allowed_symlink(session->file_on_disk, session->host->website_root)) {
-			case error:
-				log_error(session, "error while scanning file for symlinks");
-				code = 500;
-				break;
-			case not_found:
-				code = 404;
-				break;
-			case no_access:
-			case yes:
-				log_error(session, fb_symlink);
-				code = 403;
-				break;
-			case no:
-				break;
-		}
-	}
-
-	session->return_code = code;
-	if (send_header(session) == -1) {
-		return -1;
-	} else if (send_buffer(session, hs_allow, 25) == -1) {
-		return -1;
-	} else if (session->cgi_type != no_cgi) {
-		if (send_buffer(session, hs_post, 6) == -1) {
-			return -1;
-		}
-	}
-	if (session->binding->enable_trace) {
-		if (send_buffer(session, hs_trace, 7) == -1) {
-			return -1;
-		}
-	}
-	if (session->binding->enable_alter) {
-		if (send_buffer(session, hs_alter, 13) == -1) {
-			return -1;
-		}
-	}
-	if (send_buffer(session, hs_eol, 2) == -1) {
-		return -1;
-	} else if (send_buffer(session, hs_conlen, 16) == -1) {
-		return -1;
-	} else if (send_buffer(session, "0\r\n\r\n", 5) == -1) {
-		return -1;
-	}
-	session->header_sent = true;
-
-	return 200;
-}
-
 int handle_trace_request(t_session *session) {
 	int result = -1, code, body_size;
 	size_t len;
@@ -1177,6 +1109,8 @@ int handle_trace_request(t_session *session) {
 	return result;
 }
 
+/* Determine allowance of alter requests
+ */
 static t_access allow_alter(t_session *session) {
 	char *x_forwarded_for;
 	t_ip_addr forwarded_ip;
@@ -1195,6 +1129,8 @@ static t_access allow_alter(t_session *session) {
 	return unspecified;
 }
 
+/* Handle PUT requests
+ */
 int handle_put_request(t_session *session) {
 	int handle_write, handle_read = -1, result = -1, total_written = 0, lock_timeout;
 	off_t write_begin, write_end, total_size, file_size;
@@ -1379,6 +1315,8 @@ int handle_put_request(t_session *session) {
 	return result;
 }
 
+/* Handle DELETE requests
+ */
 int handle_delete_request(t_session *session) {
 	/* Access check
 	 */
