@@ -16,7 +16,7 @@
 
 int send_file(t_session *session);
 int execute_cgi(t_session *session);
-int handle_options_request(t_session *session);
+//int handle_options_request(t_session *session);
 int handle_trace_request(t_session *session);
 int handle_put_request(t_session *session);
 int handle_delete_request(t_session *session);
