@@ -265,7 +265,7 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 		new_rule->operation = to_replace;
 		new_rule->flow = tf_exit;
 
-		if (valid_uri(value) == false) {
+		if (valid_uri(value, false) == false) {
 			return false;
 		} else if ((new_rule->parameter = strdup(value)) == NULL) {
 			return false;
@@ -467,7 +467,7 @@ static int do_rewrite(char *url, regex_t *regexp, regmatch_t *pmatch, char *rep,
 }
 
 void init_toolkit_options(t_toolkit_options *options, char *website_root, t_url_toolkit *toolkit,
-	                      bool use_ssl, t_headerfield *headerfields) {
+	                      bool use_ssl, bool allow_dot_files, t_headerfield *headerfields) {
 	options->sub_depth = 0;
 	options->new_url = NULL;
 	options->website_root = website_root;
@@ -476,6 +476,7 @@ void init_toolkit_options(t_toolkit_options *options, char *website_root, t_url_
 	options->expire = -1;
 	options->url_toolkit = toolkit;
 	options->use_ssl = use_ssl;
+	options->allow_dot_files = allow_dot_files;
 	options->headerfields = headerfields;
 }
 
@@ -529,7 +530,7 @@ int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options) {
 			case tc_requesturi:
 				/* Request URI
 				 */
-				if (valid_uri(url) == false) {
+				if (valid_uri(url, false) == false) {
 					break;
 				}
 				if ((file = make_path(options->website_root, url)) == NULL) {
