@@ -63,6 +63,7 @@ typedef struct type_toolkit_options {
 	int  ban;
 	int  expire;
 	bool use_ssl;
+	bool allow_dot_files;
 	t_url_toolkit *url_toolkit;
 	t_headerfield *headerfields;
 } t_toolkit_options;
@@ -71,7 +72,7 @@ t_url_toolkit *find_toolkit(char *toolkit_id, t_url_toolkit *url_toolkit);
 bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit);
 bool toolkit_rules_oke(t_url_toolkit *url_toolkit);
 void init_toolkit_options(t_toolkit_options *options, char *website_root, t_url_toolkit *toolkit,
-                          bool use_ssl, t_headerfield *headerfields);
+                          bool use_ssl, bool allow_dot_files, t_headerfield *headerfields);
 int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options);
 
 #endif
